@@ -84,8 +84,8 @@ class Connection:
         self._session_nonce = self._getNonce()
         self._session_state = self._getState()
 
-        self._session_auth_ref_url = BASE_SESSION
-        self._session_spin_ref_url = BASE_SESSION
+        self._session_auth_ref_url = {'default': BASE_SESSION}
+        self._session_spin_ref_url = {'default': BASE_SESSION}
         self._session_first_update = False
         self._session_auth_username = username
         self._session_auth_password = password
@@ -204,7 +204,7 @@ class Connection:
             # If we need to sign in (first token)
             if 'signin-service' in ref:
                 _LOGGER.debug("Got redirect to signin-service")
-                location = await self._signin_service(req, authissuer, authorizationEndpoint)
+                location = await self._signin_service(req, authissuer, authorizationEndpoint, client)
             else:
                 # We are already logged on, shorter authorization flow
                 location = req.headers.get('Location', None)
@@ -221,7 +221,7 @@ class Connection:
                             timeout = parse_qs(urlparse(location).query).get('enableNextButtonAfterSeconds', '')[0]
                             raise SkodaAccountLockedException(f'Account is locked for another {timeout} seconds')
                         elif error == 'login.errors.password_invalid':
-                            raise SkodaAuthenticationException('Invalid credentials')
+                            raise SkodaAuthenticationException('Invalid password')
                         else:
                             _LOGGER.warning(f'Login failed: {error}')
                         raise SkodaLoginFailedException(error)
@@ -275,6 +275,7 @@ class Connection:
                     raise SkodaServiceUnavailable(f'API returned HTTP status {req.status}')
                 raise SkodaException(f'Token exchange failed. Request status: {req.status}')
             # Save access, identity and refresh tokens according to requested client
+            _LOGGER.debug('Received token exchange response, checking for errors.')
             token_data = await req.json()
             self._session_tokens[client] = {}
             for key in token_data:
@@ -318,21 +319,20 @@ class Connection:
             return False
         return True
 
-    async def _signin_service(self, html, authissuer, authorizationEndpoint):
+    async def _signin_service(self, html, authissuer, authorizationEndpoint, client='skoda'):
         """Method to signin to Skoda Connect portal."""
         # Extract login form and extract attributes
         try:
             response_data = await html.text()
             responseSoup = BeautifulSoup(response_data, 'html.parser')
-            mailform = dict()
+            form_data = dict()
             if responseSoup is None:
                 raise SkodaLoginFailedException('Login failed, server did not return a login form')
             for t in responseSoup.find('form', id='emailPasswordForm').find_all('input', type='hidden'):
                 if self._session_fulldebug:
                     _LOGGER.debug(f'Extracted form attribute: {t["name"], t["value"]}')
-                mailform[t['name']] = t['value']
-            #mailform = dict([(t['name'],t['value']) for t in responseSoup.find('form', id='emailPasswordForm').find_all('input', type='hidden')])
-            mailform['email'] = self._session_auth_username
+                form_data[t['name']] = t['value']
+            form_data['email'] = self._session_auth_username
             pe_url = authissuer+responseSoup.find('form', id='emailPasswordForm').get('action')
         except Exception as e:
             _LOGGER.error('Failed to extract user login form.')
@@ -346,7 +346,7 @@ class Connection:
         req = await self._session.post(
             url = pe_url,
             headers = self._session_auth_headers,
-            data = mailform
+            data = form_data
         )
         if req.status != 200:
             raise SkodaException('Authorization request failed')
@@ -354,16 +354,39 @@ class Connection:
             response_data = await req.text()
             responseSoup = BeautifulSoup(response_data, 'html.parser')
             pwform = {}
-            for t in responseSoup.find('form', id='credentialsForm').find_all('input', type='hidden'):
-                if self._session_fulldebug:
-                    _LOGGER.debug(f'Extracted form attribute: {t["name"], t["value"]}')
-                pwform[t['name']] = t['value']
-            #pwform = dict([(t['name'],t['value']) for t in responseSoup.find('form', id='credentialsForm').find_all('input', type='hidden')])
-            pwform['password'] = self._session_auth_password
-            pw_url = authissuer+responseSoup.find('form', id='credentialsForm').get('action')
+            credentials_form = responseSoup.find('form', id='credentialsForm')
+            all_scripts = responseSoup.find_all('script', {'src': False})
+            if credentials_form is not None:
+                _LOGGER.debug('Found HTML credentials form, extracting attributes')
+                for t in credentials_form.find_all('input', type='hidden'):
+                    if self._session_fulldebug:
+                        _LOGGER.debug(f'Extracted form attribute: {t["name"], t["value"]}')
+                    pwform[t['name']] = t['value']
+                    form_data = pwform
+                    post_action = responseSoup.find('form', id='credentialsForm').get('action')
+            elif all_scripts is not None:
+                _LOGGER.debug('Found dynamic credentials form, extracting attributes')
+                pattern = re.compile("templateModel: (.*?),\n")
+                for sc in all_scripts:
+                    if(pattern.search(sc.string)):
+                        import json
+                        data = pattern.search(sc.string)
+                        jsondata = json.loads(data.groups()[0])
+                        _LOGGER.debug(f'JSON: {jsondata}')
+                        if not jsondata.get('hmac', False):
+                            raise SkodaLoginFailedException('Failed to extract login hmac attribute')
+                        if not jsondata.get('postAction', False):
+                            raise SkodaLoginFailedException('Failed to extract login post action attribute')
+                        if jsondata.get('error', None) is not None:
+                            raise SkodaLoginFailedException(f'Login failed with error: {jsondata.get("error", None)}')
+                        form_data['hmac'] = jsondata.get('hmac', '')
+                        post_action = jsondata.get('postAction')
+            else:
+                raise SkodaLoginFailedException('Failed to extract login form data')
+            form_data['password'] = self._session_auth_password
+        except (SkodaLoginFailedException) as e:
+            raise
         except Exception as e:
-            if responseSoup.find('form', id='credentialsForm') is None:
-                raise SkodaAuthenticationException("Invalid username")
             raise SkodaAuthenticationException("Invalid username or service unavailable")
 
         # POST password
@@ -371,12 +394,18 @@ class Connection:
         self._session_auth_headers['Referer'] = pe_url
         self._session_auth_headers['Origin'] = authissuer
         _LOGGER.debug(f"Finalizing login")
+
+        client_id = CLIENT_LIST[client].get('CLIENT_ID')
+        pp_url = authissuer+post_action
+        if not 'signin-service' in pp_url or not client_id in pp_url:
+            pp_url = authissuer+'/signin-service/v1/'+client_id+"/"+post_action
+
         if self._session_fulldebug:
-            _LOGGER.debug(f'Using login action url: "{pw_url}"')
+            _LOGGER.debug(f'Using login action url: "{pp_url}"')
         req = await self._session.post(
-            url=pw_url,
+            url=pp_url,
             headers=self._session_auth_headers,
-            data = pwform,
+            data = form_data,
             allow_redirects=False
         )
         return req.headers.get('Location', None)
@@ -653,17 +682,14 @@ class Connection:
                 if 'status' in consent.get('mandatoryConsentInfo', []):
                     if consent.get('mandatoryConsentInfo', [])['status'] != 'VALID':
                         _LOGGER.error(f'The user needs to update consent for {consent.get("mandatoryConsentInfo", [])["id"]}. If problems are encountered please visit the web portal first and accept terms and conditions.')
-                        #raise SkodaEULAException(f'User needs to update consent for {consent.get("mandatoryConsentInfo", [])["id"]}')
                 elif len(consent.get('missingMandatoryFields', [])) > 0:
                     _LOGGER.error(f'Missing mandatory field for user: {consent.get("missingMandatoryFields", [])[0].get("name", "")}. If problems are encountered please visit the web portal first and accept terms and conditions.')
-                    #raise SkodaEULAException(f'Missing mandatory field for user: {consent.get("missingMandatoryFields", [])[0].get("name", "")}')
                 else:
                     _LOGGER.debug('User consent is valid, no missing information for profile')
             else:
                 _LOGGER.debug('Could not fetch consent information. If problems are encountered please visit the web portal first and make sure that no new terms and conditions need to be accepted.')
         except:
             _LOGGER.debug('Could not fetch consent information. If problems are encountered please visit the web portal first and make sure that no new terms and conditions need to be accepted.')
-            #raise
 
         # Authorize for "skoda" client and get vehicles from garage endpoint
         try:
@@ -761,7 +787,7 @@ class Connection:
             # Try new pyJWT syntax if old fails
             if subject is None:
                 try:
-                    exp = jwt.decode(atoken, options={'verify_signature': False}).get('sub', None)
+                    subject = jwt.decode(atoken, options={'verify_signature': False}).get('sub', None)
                 except:
                     raise Exception("Could not extract sub attribute from token")
 
@@ -820,11 +846,13 @@ class Connection:
         try:
             await self.set_token('vwg')
             response = await self.get(f'https://mal-1a.prd.ece.vwg-connect.com/api/cs/vds/v1/vehicles/{vin}/homeRegion', vin)
-            self._session_auth_ref_url = response['homeRegion']['baseUri']['content'].split('/api')[0].replace('mal-', 'fal-') if response['homeRegion']['baseUri']['content'] != 'https://mal-1a.prd.ece.vwg-connect.com/api' else 'https://msg.volkswagen.de'
-            self._session_spin_ref_url = response['homeRegion']['baseUri']['content'].split('/api')[0]
+            self._session_auth_ref_url[vin] = response['homeRegion']['baseUri']['content'].split('/api')[0].replace('mal-', 'fal-') if response['homeRegion']['baseUri']['content'] != 'https://mal-1a.prd.ece.vwg-connect.com/api' else 'https://msg.volkswagen.de'
+            self._session_spin_ref_url[vin] = response['homeRegion']['baseUri']['content'].split('/api')[0]
             return response['homeRegion']['baseUri']['content']
         except Exception as error:
             _LOGGER.debug(f'Could not get homeregion, error {error}')
+            self._session_auth_ref_url[vin] = self._session_auth_ref_url['default']
+            self._session_spin_ref_url[vin] = self._session_spin_ref_url['default']
         return False
 
     async def getOperationList(self, vin):
@@ -833,7 +861,7 @@ class Connection:
             await self.set_token('vwg')
             response = await self.get(
                 urljoin(
-                    self._session_spin_ref_url,
+                    self._session_spin_ref_url[vin],
                     f'/api/rolesrights/operationlist/v3/vehicles/{vin}'
                 )
             )
@@ -890,7 +918,7 @@ class Connection:
             await self.set_token('vwg')
             response = await self.get(
                 urljoin(
-                    self._session_auth_ref_url,
+                    self._session_auth_ref_url[vin],
                     f'fs-car/bs/vsr/v1/{BRAND}/{COUNTRY}/vehicles/{vin}/status'
                 )
             )
@@ -914,7 +942,7 @@ class Connection:
             await self.set_token('smartlink')
             response = await self.get(
                 urljoin(
-                    self._session_auth_ref_url,
+                    self._session_auth_ref_url[vin],
                     f'https://api.connect.skoda-auto.cz/api/v1/vehicle-status/{vin}'
                 )
             )
@@ -937,7 +965,7 @@ class Connection:
             await self.set_token('vwg')
             response = await self.get(
                 urljoin(
-                    self._session_auth_ref_url,
+                    self._session_auth_ref_url[vin],
                     f'fs-car/bs/tripstatistics/v1/{BRAND}/{COUNTRY}/vehicles/{vin}/tripdata/shortTerm?newest'
                 )
             )
@@ -958,7 +986,7 @@ class Connection:
             await self.set_token('vwg')
             response = await self.get(
                 urljoin(
-                    self._session_auth_ref_url,
+                    self._session_auth_ref_url[vin],
                     f'fs-car/bs/cf/v1/{BRAND}/{COUNTRY}/vehicles/{vin}/position'
                 )
             )
@@ -990,7 +1018,7 @@ class Connection:
             await self.set_token('vwg')
             response = await self.get(
                 urljoin(
-                    self._session_auth_ref_url,
+                    self._session_auth_ref_url[vin],
                     f'fs-car/bs/departuretimer/v1/{BRAND}/{COUNTRY}/vehicles/{vin}/timer'
                 )
             )
@@ -1027,7 +1055,7 @@ class Connection:
             await self.set_token('vwg')
             response = await self.get(
                 urljoin(
-                    self._session_auth_ref_url,
+                    self._session_auth_ref_url[vin],
                     f'fs-car/bs/climatisation/v1/{BRAND}/{COUNTRY}/vehicles/{vin}/climater'
                 )
             )
@@ -1072,7 +1100,7 @@ class Connection:
             await self.set_token('vwg')
             response = await self.get(
                 urljoin(
-                    self._session_auth_ref_url,
+                    self._session_auth_ref_url[vin],
                     f'fs-car/bs/batterycharge/v1/{BRAND}/{COUNTRY}/vehicles/{vin}/charger'
                 )
             )
@@ -1113,7 +1141,7 @@ class Connection:
             await self.set_token('vwg')
             response = await self.get(
                 urljoin(
-                    self._session_auth_ref_url,
+                    self._session_auth_ref_url[vin],
                     f'fs-car/bs/rs/v1/{BRAND}/{COUNTRY}/vehicles/{vin}/status'
                 )
             )
@@ -1160,7 +1188,7 @@ class Connection:
 
             response = await self.get(
                 urljoin(
-                    self._session_auth_ref_url,
+                    self._session_auth_ref_url[vin],
                     url
                 )
             )
@@ -1217,7 +1245,7 @@ class Connection:
                 raise SkodaException(f'Security token for "{action}" is not implemented')
             response = await self.get(
                 urljoin(
-                    self._session_spin_ref_url,
+                    self._session_spin_ref_url[vin],
                     urls.get(action)
                 )
             )
@@ -1234,7 +1262,13 @@ class Connection:
                 }
             }
             self._session_headers['Content-Type'] = 'application/json'
-            response = await self.post(urljoin(self._session_spin_ref_url, '/api/rolesrights/authorization/v2/security-pin-auth-completed'), json = body)
+            response = await self.post(
+                urljoin(
+                    self._session_spin_ref_url[vin],
+                    '/api/rolesrights/authorization/v2/security-pin-auth-completed'
+                ),
+                json = body
+            )
             self._session_headers.pop('Content-Type', None)
             if response.get('securityToken', False):
                 return response['securityToken']
@@ -1245,17 +1279,18 @@ class Connection:
             raise
 
    # VW-Group API methods
-    async def _setVWAPI(self, endpoint, **data):
+    async def _setVWAPI(self, url, **data): #endpoint, **data):
         """Data call through VW-Group API."""
         try:
             await self.set_token('vwg')
             # Combine homeregion with endpoint URL
-            url = urljoin(self._session_auth_ref_url, endpoint)
+            #url = urljoin(self._session_auth_ref_url, endpoint)
             response = await self._data_call(url, **data)
             self._session_headers.pop('X-mbbSecToken', None)
             self._session_headers.pop('X-securityToken', None)
             if not response:
-                raise SkodaException(f'Invalid or no response for endpoint {endpoint}')
+                #raise SkodaException(f'Invalid or no response for endpoint {endpoint}')
+                raise SkodaException(f'Invalid or no response for endpoint {url}')
             elif response == 429:
                 raise SkodaThrottledException('Action rate limit reached. Start the car to reset the action limit')
             else:
@@ -1283,7 +1318,13 @@ class Connection:
 
     async def setCharger(self, vin, data):
         """Start/Stop charger."""
-        return await self._setVWAPI(f'fs-car/bs/batterycharge/v1/{BRAND}/{COUNTRY}/vehicles/{vin}/charger/actions', json = data)
+        return await self._setVWAPI(
+            urljoin(
+                self._session_auth_ref_url[vin],
+                f'fs-car/bs/batterycharge/v1/{BRAND}/{COUNTRY}/vehicles/{vin}/charger/actions'
+            ),
+            json = data
+        )
 
     async def setClimater(self, vin, data, spin):
         """Execute climatisation actions."""
@@ -1291,7 +1332,13 @@ class Connection:
             # Only get security token if auxiliary heater is to be started
             if data.get('action', {}).get('settings', {}).get('heaterSource', None) == 'auxiliary':
                 self._session_headers['X-securityToken'] = await self.get_sec_token(vin = vin, spin = spin, action = 'rclima')
-            return await self._setVWAPI(f'fs-car/bs/climatisation/v1/{BRAND}/{COUNTRY}/vehicles/{vin}/climater/actions', json = data)
+            return await self._setVWAPI(
+                urljoin(
+                self._session_auth_ref_url[vin],
+                f'fs-car/bs/climatisation/v1/{BRAND}/{COUNTRY}/vehicles/{vin}/climater/actions'
+            ),
+            json = data
+        )
         except:
             raise
         return False
@@ -1401,14 +1448,28 @@ class Connection:
             # Only get security token if auxiliary heater is to be enabled
             #if data.get... == 'auxiliary':
             #   self._session_headers['X-securityToken'] = await self.get_sec_token(vin = vin, spin = spin, action = 'timer')
-            return await self._setVWAPI(f'fs-car/bs/departuretimer/v1/{BRAND}/{COUNTRY}/vehicles/{vin}/timer/actions', json = body)
+            #return await self._setVWAPI(
+            return await self._setVWAPI(
+                urljoin(
+                    self._session_auth_ref_url[vin],
+                    f'fs-car/bs/departuretimer/v1/{BRAND}/{COUNTRY}/vehicles/{vin}/timer/actions'
+                ),
+                json = body
+            )
         except:
             raise
         return False
 
     async def setHonkAndFlash(self, vin, data):
         """Execute honk and flash actions."""
-        return await self._setVWAPI(f'fs-car/bs/rhf/v1/{BRAND}/{COUNTRY}/vehicles/{vin}/honkAndFlash', json = data)
+        #return await self._setVWAPI(
+        return await self._setVWAPI(
+            urljoin(
+                self._session_auth_ref_url[vin],
+                f'fs-car/bs/rhf/v1/{BRAND}/{COUNTRY}/vehicles/{vin}/honkAndFlash'
+            ),
+            json = data
+        )
 
     async def setLock(self, vin, data, spin):
         """Remote lock and unlock actions."""
@@ -1426,7 +1487,13 @@ class Connection:
             # Set temporary Content-Type
             self._session_headers['Content-Type'] = 'application/vnd.vwg.mbb.RemoteLockUnlock_v1_0_0+xml'
 
-            response = await self._setVWAPI(f'fs-car/bs/rlu/v1/{BRAND}/{COUNTRY}/vehicles/{vin}/actions', data = data)
+            response = await self._setVWAPI(
+                urljoin(
+                    self._session_auth_ref_url[vin],
+                    f'fs-car/bs/rlu/v1/{BRAND}/{COUNTRY}/vehicles/{vin}/actions'
+                ),
+                data = data
+            )
 
             # Clean up content-type
             self._session_headers.pop('Content-Type', None)
@@ -1453,7 +1520,13 @@ class Connection:
                     self._session_headers['x-mbbSecToken'] = await self.get_sec_token(vin = vin, spin = spin, action = 'heating')
             else:
                 raise SkodaConfigException("Invalid data for preheater")
-            response = await self._setVWAPI(f'fs-car/bs/rs/v1/{BRAND}/{COUNTRY}/vehicles/{vin}/action', json = data)
+            response = await self._setVWAPI(
+                urljoin(
+                    self._session_auth_ref_url[vin],
+                    f'fs-car/bs/rs/v1/{BRAND}/{COUNTRY}/vehicles/{vin}/action'
+                ),
+                json = data
+            )
 
             # Clean up headers
             self._session_headers.pop('x-mbbSecToken', None)
@@ -1471,7 +1544,13 @@ class Connection:
 
     async def setRefresh(self, vin):
         """"Force vehicle data update."""
-        return await self._setVWAPI(f'fs-car/bs/vsr/v1/{BRAND}/{COUNTRY}/vehicles/{vin}/requests', data=None)
+        return await self._setVWAPI(
+            urljoin(
+                self._session_auth_ref_url[vin],
+                f'fs-car/bs/vsr/v1/{BRAND}/{COUNTRY}/vehicles/{vin}/requests'
+            ),
+            data=None
+        )
 
    # Skoda native API request methods
     async def _setSkodaAPI(self, endpoint, vin, **data):
