@@ -87,7 +87,7 @@ class Sensor(Instrument):
                 self.unit = "mi/h"
                 self.convert = True
             elif "l/100 km" == self.unit:
-                self.unit = "l/100 mi"
+                self.unit = "mpg"
                 self.convert = True
             elif "kWh/100 km" == self.unit:
                 self.unit = "kWh/100 mi"
@@ -98,9 +98,9 @@ class Sensor(Instrument):
             elif "km/h" == self.unit:
                 self.unit = "mil/h"
             elif "l/100 km" == self.unit:
-                self.unit = "l/100 mil"
+                self.unit = "l/mil"
             elif "kWh/100 km" == self.unit:
-                self.unit = "kWh/100 mil"
+                self.unit = "kWh/mil"
 
         # Init placeholder for parking heater duration
         config.get('parkingheater', 30)
@@ -126,14 +126,14 @@ class Sensor(Instrument):
             return int(round(val / 1.609344))
         elif val and self.unit and "mi/h" in self.unit and self.convert is True:
             return int(round(val / 1.609344))
-        elif val and self.unit and "gal/100 mi" in self.unit and self.convert is True:
-            return round(val * 0.4251438, 1)
+        elif val and self.unit and "mpg" in self.unit and self.convert is True:
+            return round(val * 235.215, 1)
         elif val and self.unit and "kWh/100 mi" in self.unit and self.convert is True:
             return round(val * 0.4251438, 1)
         elif val and self.unit and "°F" in self.unit and self.convert is True:
             temp = round((val * 9 / 5) + 32, 1)
             return temp
-        elif val and self.unit in ['mil', 'mil/h']:
+        elif val and self.unit in ['mil', 'mil/h', 'l/mil', 'kWh/mil']:
             return val / 10
         else:
             return val
@@ -359,7 +359,10 @@ class DoorLock(Instrument):
 
     @property
     def attributes(self):
-        return dict(last_result = self.vehicle.lock_action_status)
+        return {
+            'last_result': self.vehicle.lock_action_status,
+            'last_timestamp': self.vehicle.lock_action_timestamp
+        }
 
 
 class TrunkLock(Instrument):
@@ -412,7 +415,10 @@ class RequestHonkAndFlash(Switch):
 
     @property
     def attributes(self):
-        return dict(last_result = self.vehicle.honkandflash_action_status)
+        return {
+            'last_result': self.vehicle.honkandflash_action_status,
+            'last_timestamp': self.vehicle.honkandflash_action_timestamp
+        }
 
 
 class RequestFlash(Switch):
@@ -438,7 +444,10 @@ class RequestFlash(Switch):
 
     @property
     def attributes(self):
-        return dict(last_result = self.vehicle.honkandflash_action_status)
+        return {
+            'last_result': self.vehicle.honkandflash_action_status,
+            'last_timestamp': self.vehicle.honkandflash_action_timestamp
+        }
 
 
 class RequestUpdate(Switch):
@@ -447,7 +456,10 @@ class RequestUpdate(Switch):
 
     @property
     def state(self):
-        return self.vehicle.refresh_data
+        if self.vehicle.refresh_data is None:
+            return False
+        else:
+            return self.vehicle.refresh_data
 
     async def turn_on(self):
         await self.vehicle.set_refresh()
@@ -464,7 +476,10 @@ class RequestUpdate(Switch):
 
     @property
     def attributes(self):
-        return dict(last_result = self.vehicle.refresh_action_status)
+        return {
+            'last_result': self.vehicle.refresh_action_status,
+            'last_timestamp': self.vehicle.refresh_action_timestamp
+        }
 
 
 class ElectricClimatisation(Switch):
@@ -493,8 +508,10 @@ class ElectricClimatisation(Switch):
         if self.vehicle.is_electric_climatisation_attributes_supported:
             attrs = self.vehicle.electric_climatisation_attributes
             attrs['last_result'] = self.vehicle.climater_action_status
+            attrs['last_timestamp'] = self.vehicle.climater_action_timestamp
         else:
             attrs['last_result'] = self.vehicle.climater_action_status
+            attrs['last_timestamp'] = self.vehicle.climater_action_timestamp
         return attrs
 
 
@@ -523,7 +540,10 @@ class AuxiliaryClimatisation(Switch):
 
     @property
     def attributes(self):
-        return dict(last_result = self.vehicle.climater_action_status)
+        return {
+            'last_result': self.vehicle.climater_action_status,
+            'last_timestamp': self.vehicle.climater_action_timestamp
+        }
 
 
 class Charging(Switch):
@@ -548,7 +568,10 @@ class Charging(Switch):
 
     @property
     def attributes(self):
-        return dict(last_result = self.vehicle.charger_action_status)
+        return {
+            'last_result': self.vehicle.charger_action_status,
+            'last_timestamp': self.vehicle.charger_action_timestamp
+        }
 
 
 class WindowHeater(Switch):
@@ -573,10 +596,10 @@ class WindowHeater(Switch):
 
     @property
     def attributes(self):
-        return dict(
-            last_result = self.vehicle.climater_action_status,
-
-        )
+        return {
+            'last_result': self.vehicle.climater_action_status,
+            'last_timestamp': self.vehicle.climater_action_timestamp
+        }
 
 
 class SeatHeatingFrontLeft(Switch):
@@ -603,7 +626,10 @@ class SeatHeatingFrontLeft(Switch):
 
     @property
     def attributes(self):
-        return dict(last_result = self.vehicle.climater_action_status)
+        return {
+            'last_result': self.vehicle.aircon_action_status,
+            'last_timestamp': self.vehicle.aircon_action_timestamp
+        }
 
 class SeatHeatingFrontRight(Switch):
     def __init__(self):
@@ -629,7 +655,10 @@ class SeatHeatingFrontRight(Switch):
 
     @property
     def attributes(self):
-        return dict(last_result = self.vehicle.climater_action_status)
+        return {
+            'last_result': self.vehicle.aircon_action_status,
+            'last_timestamp': self.vehicle.aircon_action_timestamp
+        }
 
 class SeatHeatingRearLeft(Switch):
     def __init__(self):
@@ -655,7 +684,10 @@ class SeatHeatingRearLeft(Switch):
 
     @property
     def attributes(self):
-        return dict(last_result = self.vehicle.climater_action_status)
+        return {
+            'last_result': self.vehicle.aircon_action_status,
+            'last_timestamp': self.vehicle.aircon_action_timestamp
+        }
 
 class SeatHeatingRearRight(Switch):
     def __init__(self):
@@ -681,7 +713,10 @@ class SeatHeatingRearRight(Switch):
 
     @property
     def attributes(self):
-        return dict(last_result = self.vehicle.climater_action_status)
+        return {
+            'last_result': self.vehicle.aircon_action_status,
+            'last_timestamp': self.vehicle.aircon_action_timestamp
+        }
 
 
 class AirConditionAtUnlock(Switch):
@@ -704,7 +739,10 @@ class AirConditionAtUnlock(Switch):
 
     @property
     def attributes(self):
-        return dict(last_result = self.vehicle.climater_action_status)
+        return {
+            'last_result': self.vehicle.aircon_action_status,
+            'last_timestamp': self.vehicle.aircon_action_timestamp
+        }
 
 
 class BatteryClimatisation(Switch):
@@ -729,7 +767,10 @@ class BatteryClimatisation(Switch):
 
     @property
     def attributes(self):
-        return dict(last_result = self.vehicle.climater_action_status)
+        return {
+            'last_result': self.vehicle.climater_action_status,
+            'last_timestamp': self.vehicle.climater_action_timestamp
+        }
 
 
 class PHeaterHeating(Switch):
@@ -758,7 +799,10 @@ class PHeaterHeating(Switch):
 
     @property
     def attributes(self):
-        return dict(last_result = self.vehicle.pheater_action_status)
+        return {
+            'last_result': self.vehicle.pheater_action_status,
+            'last_timestamp': self.vehicle.pheater_action_timestamp
+        }
 
 
 class PHeaterVentilation(Switch):
@@ -787,8 +831,10 @@ class PHeaterVentilation(Switch):
 
     @property
     def attributes(self):
-        return dict(last_result = self.vehicle.pheater_action_status)
-
+        return {
+            'last_result': self.vehicle.pheater_action_status,
+            'last_timestamp': self.vehicle.pheater_action_timestamp
+        }
 
 class DepartureTimer1(Switch):
     def __init__(self):
@@ -912,6 +958,69 @@ class RequestResults(Sensor):
     def attributes(self):
         return dict(self.vehicle.request_results)
 
+class ChargingPower(Sensor):
+    def __init__(self):
+        super().__init__(attr="charging_power", name="Charging Power", icon="mdi:flash", unit="W", device_class="power")
+
+    @property
+    def state(self):
+        return self.vehicle.charging_power
+
+    @property
+    def assumed_state(self):
+        return False
+
+    @property
+    def attributes(self):
+        values = {
+            "max_power": self.vehicle.max_charging_power
+        }
+        return values
+
+class CarInfo(Sensor):
+    def __init__(self):
+        super().__init__(attr="model", name="Car Info", icon="mdi:information-outline", unit=None)
+
+    @property
+    def state(self):
+        return self.vehicle.vin
+
+    @property
+    def assumed_state(self):
+        return False
+
+    @property
+    def attributes(self):
+        values = {}
+        if self.vehicle.is_engine_capacity_supported:
+            values['engine'] = {
+                "type": self.vehicle.engine_type,
+                "power_in_kw": self.vehicle.engine_power,
+                "capacity_in_liter": self.vehicle.engine_capacity
+            }
+        else:
+            if self.vehicle.is_battery_capacity_supported:
+                values['engine'] = {
+                    "engine_type": self.vehicle.engine_type,
+                    "power_in_kw": self.vehicle.engine_power,
+                    "battery_capacity_kwh": self.vehicle.battery_capacity
+                }
+            else:
+                values['engine'] = {
+                    "engine_type": self.vehicle.engine_type,
+                    "power_in_kw": self.vehicle.engine_power
+                }
+
+        values['model'] = {
+            "model": self.vehicle.model,
+            "manufactured": self.vehicle.model_year
+        }
+        if self.vehicle.is_model_image_large_supported:
+            values['model']['image_url_large'] = self.vehicle.model_image_large
+        if self.vehicle.is_model_image_small_supported:
+            values['model']['image_url_small'] = self.vehicle.model_image_small
+
+        return values
 
 def create_instruments():
     return [
@@ -934,8 +1043,10 @@ def create_instruments():
         PHeaterHeating(),
         #ElectricClimatisationClimate(),
         #CombustionClimatisationClimate(),
+        CarInfo(),
         Charging(),
         RequestResults(),
+        ChargingPower(),
         DepartureTimer1(),
         DepartureTimer2(),
         DepartureTimer3(),
@@ -1013,13 +1124,6 @@ def create_instruments():
             icon="mdi:battery-charging-100",
             unit="h",
         ),
-        Sensor(
-            attr="charging_power",
-            name="Charging power",
-            icon="mdi:flash",
-            unit="W",
-            device_class="power"
-        ),
         Sensor(
             attr="charge_rate",
             name="Charging rate",
@@ -1307,7 +1411,7 @@ def create_instruments():
         BinarySensor(
             attr="aircon_at_unlock",
             name="Air-conditioning at unlock",
-            device_class="opening"
+            device_class=None
         ),
     ]
 
